require 'oss/version'
require 'oss/api'
require 'oss/client'
require 'oss/service'
require 'oss/bucket'
require 'oss/object'
require 'oss/multipart'
require 'oss/cors'

module Oss

  attr_reader :config

  # load configs for Rails app
  def self.config
    @config ||= begin
      if defined? Rails
        config_file = Rails.root.join('config/oss.yml')
        config = YAML.load(ERB.new(File.new(config_file).read).result)[Rails.env] if File.exist?(config_file)
      end

      config ||= config_from_environment
      config.symbolize_keys!
      config[:endpoint] ||= 'oss-cn-hangzhou.aliyuncs.com'
      OpenStruct.new(config)
    end
  end

  def self.api
    @api ||= Api.new(config.endpoint, config.access_key_id, config.access_key_secret, config.bucket)
  end

  # load configs from environment
  def self.config_from_environment
    {
        access_key_id: ENV['OSS_ID'],
        access_key_secret: ENV['OSS_SECRET'],
        endpoint: ENV['ENDPOINT'],
        bucket: ENV['BUCKET'],
    }
  end

end
