require 'oss/service'
require 'oss/bucket'
require 'oss/object'
require 'oss/multipart'
require 'oss/cors'

module Oss

  class Api

    attr_accessor :client, :bucket

    def initialize(endpoint, access_key_id, access_key_secret, bucket = nil)
      @bucket = bucket
      @client = Client.new(endpoint, access_key_id, access_key_secret)
    end


    #######################
    # Service
    #######################

    # params:
    # - options:
    #   - prefix
    #   - marker
    #   - max_keys
    def get_service(options = {})
      Service.new(@client).get_service(options)
    end

    alias_method :list_bucket, :get_service


    #######################
    # Bucket
    #######################

    # params:
    # - bucket_name
    # - acl
    def put_bucket(bucket_name, acl)
      Bucket.new(@client).put_bucket(bucket_name, acl)
    end

    # params:
    # - bucket_name
    # - acl
    def put_bucket_acl(bucket_name, acl)
      Bucket.new(@client).put_bucket_acl(bucket_name, acl)
    end

    # params:
    # - bucket_name
    # - target_bucket
    # - enable
    # - target_prefix
    def put_bucket_logging(bucket_name, target_bucket, enable = true, target_prefix = nil)
      Bucket.new(@client).put_bucket_logging(bucket_name, target_bucket, enable, target_prefix)
    end

    # params:
    # - bucket_name
    # - index_doc
    # - error_doc
    def put_bucket_website(bucket_name, index_doc, error_doc)
      Bucket.new(@client).put_bucket_website(bucket_name, index_doc, error_doc)
    end

    # params:
    # - bucket_name
    # - allow_empty
    # - referer_list
    def put_bucket_referer(bucket_name, allow_empty, referer_list = [])
      Bucket.new(@client).put_bucket_referer(bucket_name, allow_empty, referer_list)
    end

    # params:
    # - bucket_name
    # - allow_empty
    # - referer_list
    def put_bucket_lifecycle(bucket_name, rules = [])
      Bucket.new(@client).put_bucket_lifecycle(bucket_name, rules)
    end

    # params:
    # - bucket_name
    # - options:
    #   - delimiter
    #   - prefix
    #   - marker
    #   - max_keys
    #   - encoding_type
    def get_bucket(bucket_name, options = {})
      Bucket.new(@client).get_bucket(bucket_name, options)
    end

    # params:
    # - bucket_name
    def get_bucket_acl(bucket_name)
      Bucket.new(@client).get_bucket_acl(bucket_name)
    end

    # params:
    # - bucket_name
    def get_bucket_location(bucket_name)
      Bucket.new(@client).get_bucket_location(bucket_name)
    end

    # params:
    # - bucket_name
    def get_bucket_logging(bucket_name)
      Bucket.new(@client).get_bucket_logging(bucket_name)
    end

    # params:
    # - bucket_name
    def get_bucket_website(bucket_name)
      Bucket.new(@client).get_bucket_website(bucket_name)
    end

    # params:
    # - bucket_name
    def get_bucket_referer(bucket_name)
      Bucket.new(@client).get_bucket_referer(bucket_name)
    end

    # params:
    # - bucket_name
    def get_bucket_lifecycle(bucket_name)
      Bucket.new(@client).get_bucket_lifecycle(bucket_name)
    end

    # params:
    # - bucket_name
    def delete_bucket(bucket_name)
      Bucket.new(@client).delete_bucket(bucket_name)
    end

    # params:
    # - bucket_name
    def delete_bucket_logging(bucket_name)
      Bucket.new(@client).delete_bucket_logging(bucket_name)
    end

    # params:
    # - bucket_name
    def delete_bucket_website(bucket_name)
      Bucket.new(@client).delete_bucket_website(bucket_name)
    end

    # params:
    # - bucket_name
    def delete_bucket_lifecycle(bucket_name)
      Bucket.new(@client).delete_bucket_lifecycle(bucket_name)
    end


    #######################
    # Object
    #######################

    # params:
    # - bucket_name
    # - object_name
    # - file
    # - options
    def put_object(bucket_name, object_name, file, options = {})
      Object.new(@client).put_object(bucket_name, object_name, file, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - file
    # - options
    def copy_object(bucket_name, object, old_bucket, old_object, options = {})
      Object.new(@client).copy_object(bucket_name, object, old_bucket, old_object, options)
    end

    # params:
    # - bucket_name
    # - object
    # - options
    def get_object(bucket_name, object, options = {})
      Object.new(@client).get_object(bucket_name, object, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - file
    # - options
    def append_object(bucket_name, object_name, file, position, options = {})
      Object.new(@client).append_object(bucket_name, object_name, file, position, options)
    end

    # params:
    # - bucket_name
    # - object_name
    def delete_object(bucket_name, object_name)
      Object.new(@client).delete_object(bucket_name, object_name)
    end

    # params:
    # - bucket_name
    # - objects
    def delete_multiple_objects(bucket_name, objects)
      Object.new(@client).delete_multiple_objects(bucket_name, objects)
    end

    # params:
    # - bucket_name
    # - object_name
    # - options
    def head_object(bucket_name, object_name)
      Object.new(@client).head_object(bucket_name, object_name)
    end

    # params:
    # - bucket_name
    # - object_name
    # - acl
    def put_object_acl(bucket_name, object_name, acl)
      Object.new(@client).put_object_acl(bucket_name, object_name, acl)
    end

    # params:
    # - bucket_name
    # - object_name
    # - acl
    def post_object(bucket_name, key, options = {})
      Object.new(@client).post_object(bucket_name, key, options)
    end


    #######################
    # Multipart
    #######################

    # params:
    # - bucket_name
    # - object_name
    # - options
    def initiate_multipart_upload(bucket_name, object_name, options = {})
      Multipart.new(@client).initiate_multipart_upload(bucket_name, object_name, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - upload_id
    # - file
    # - part_number
    # - options
    def upload_part(bucket_name, object_name, upload_id, file, part_number = 1, options = {})
      Multipart.new(@client).upload_part(bucket_name, object_name, upload_id, file, part_number, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - upload_id
    # - old_bucket
    # - old_object
    # - part_number
    # - options
    def upload_part_copy(bucket_name, object_name, upload_id, old_bucket, old_object, part_number = 1, options = {})
      Multipart.new(@client).upload_part_copy(bucket_name, object_name, upload_id, old_bucket, old_object, part_number, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - upload_id
    # - parts
    def complete_multipart_upload(bucket_name, object_name, upload_id, parts)
      Multipart.new(@client).complete_multipart_upload(bucket_name, object_name, upload_id, parts)
    end

    # params:
    # - bucket_name
    # - object_name
    # - upload_id
    def abort_multipart_upload(bucket_name, object_name, upload_id)
      Multipart.new(@client).abort_multipart_upload(bucket_name, object_name, upload_id)
    end

    # params:
    # - bucket_name
    # - options
    def list_multipart_upload(bucket_name, options = {})
      Multipart.new(@client).list_multipart_upload(bucket_name, options)
    end

    # params:
    # - bucket_name
    # - object_name
    # - options
    def list_parts(bucket_name, object_name, options = {})
      Multipart.new(@client).list_parts(bucket_name, object_name, options)
    end


    #######################
    # Cors
    #######################

    # params:
    # - bucket_name
    # - rules
    def put_bucket_cors(bucket_name, rules)
      Cors.new(@client).put_bucket_cors(bucket_name, rules)
    end

    # params:
    # - bucket_name
    # - rules
    def get_bucket_cors(bucket_name)
      Cors.new(@client).get_bucket_cors(bucket_name)
    end

    # params:
    # - bucket_name
    # - rules
    def delete_bucket_cors(bucket_name)
      Cors.new(@client).delete_bucket_cors(bucket_name)
    end

    # params:
    # - bucket_name
    # - object_name
    # - origin
    # - request_method
    # - request_headers
    def option_object(bucket_name, object_name, origin, request_method, request_headers)
      Cors.new(@client).option_object(bucket_name, object_name, origin, request_method, request_headers)
    end

  end

end