require 'oss/client'
require 'oss/util'

module Oss

  SERVICE_HOST = 'oss.aliyuncs.com'

  class Service

    include Util

    attr_accessor :client, :xml_obj

    def initialize(client)
      @client = client
    end

    # params:
    # - options:
    #   - prefix
    #   - marker
    #   - max_keys
    def get_service(options = {})
      @xml_obj = client.get(host: SERVICE_HOST, path: '/', query_string: options).xpath('ListAllMyBucketsResult')
      self
    end

    def owner
      owner = @xml_obj.xpath('Owner')
      {
          :id           => owner.xpath('ID').text,
          :display_name => owner.xpath('DisplayName').text
      }
    end

    def buckets
      buckets = @xml_obj.xpath('Buckets')
      results = Array.new
      buckets.each do |bucket|
        results << {
            :location      => bucket.xpath('Bucket/Location').text,
            :name          => bucket.xpath('Bucket/Name').text,
            :creation_date => bucket.xpath('Bucket/CreationDate').text
        }
      end
      results
    end

    # prefix, marker, max_keys, is_truncated, next_marker
    def method_missing(method)
      if @xml_obj.nil?
        super
      else
        camel = Util.camelize(method)
        value = @xml_obj.xpath(camel)
        raise "missing xml attribute #{camel}" if value.length == 0
        value.inner_text
      end
    end

  end

end